@@ -5,7 +5,6 @@
 // The firewall rules defined by the user are reloaded in these cases:
 // - When the file system-fw.json changes.
 // - When the firewall rules are not present when listing them.
-//
 package config
 
 import (
@@ -59,21 +58,18 @@ type Expressions struct {
 
 // FwRule holds the fields of a rule
 type FwRule struct {
+	*sync.RWMutex
 	// we need to keep old fields in the struct. Otherwise when receiving a conf from the GUI, the legacy rules would be deleted.
-	Chain      string // TODO: deprecated, remove
-	Table      string // TODO: deprecated, remove
-	Parameters string // TODO: deprecated: remove
-
+	Chain            string // TODO: deprecated, remove
+	Table            string // TODO: deprecated, remove
+	Parameters       string // TODO: deprecated, remove
 	UUID             string
 	Description      string
-	Expressions      []*Expressions
 	Target           string
 	TargetParameters string
-
-	Position uint64 `json:",string"`
-	Enabled  bool
-
-	*sync.RWMutex
+	Expressions      []*Expressions
+	Position         uint64 `json:",string"`
+	Enabled          bool
 }
 
 // FwChain holds the information that defines a firewall chain.
@@ -102,33 +98,31 @@ type rulesList struct {
 }
 
 type chainsList struct {
-	Chains []*FwChain
 	Rule   *FwRule // TODO: deprecated, remove
+	Chains []*FwChain
 }
 
 // SystemConfig holds the list of rules to be added to the system
 type SystemConfig struct {
-	sync.RWMutex
 	SystemRules []*chainsList
-	Version     uint32
-	Enabled     bool
+	sync.RWMutex
+	Version uint32
+	Enabled bool
 }
 
 // Config holds the functionality to re/load the firewall configuration from disk.
 // This is the configuration to manage the system firewall (iptables, nftables).
 type Config struct {
-	sync.Mutex
-	file            string
 	watcher         *fsnotify.Watcher
 	monitorExitChan chan bool
-	SysConfig       SystemConfig
-
-	// preloadCallback is called before reloading the configuration,
-	// in order to delete old fw rules.
+	// preload will be called after daemon startup, whilst reload when a modification is performed.
 	preloadCallback func()
 	// reloadCallback is called after the configuration is written.
 	reloadCallback func()
-	// preload will be called after daemon startup, whilst reload when a modification is performed.
+	file           string
+	SysConfig      SystemConfig
+
+	sync.Mutex
 }
 
 // NewSystemFwConfig initializes config fields
@@ -151,37 +145,45 @@ func (c *Config) NewSystemFwConfig(preLoadCb, reLoadCb func()) (*Config, error)
 	return c, nil
 }
 
+func (c *Config) SetFile(file string) {
+	c.file = file
+}
+
 // LoadDiskConfiguration reads and loads the firewall configuration from disk
-func (c *Config) LoadDiskConfiguration(reload bool) {
+func (c *Config) LoadDiskConfiguration(reload bool) error {
 	c.Lock()
 	defer c.Unlock()
 
 	raw, err := ioutil.ReadFile(c.file)
 	if err != nil {
 		log.Error("Error reading firewall configuration from disk %s: %s", c.file, err)
-		return
+		return err
 	}
 
-	c.loadConfiguration(raw)
+	if err = c.loadConfiguration(raw); err != nil {
+		return err
+	}
 	// we need to monitor the configuration file for changes, regardless if it's
 	// malformed or not.
 	c.watcher.Remove(c.file)
 	if err := c.watcher.Add(c.file); err != nil {
 		log.Error("Could not watch firewall configuration: %s", err)
-		return
+		return err
 	}
 
 	if reload {
 		c.reloadCallback()
-		return
+		return nil
 	}
 
 	go c.monitorConfigWorker()
+
+	return nil
 }
 
 // loadConfigutation reads the system firewall rules from disk.
 // Then the rules are added based on the configuration defined.
-func (c *Config) loadConfiguration(rawConfig []byte) {
+func (c *Config) loadConfiguration(rawConfig []byte) error {
 	c.SysConfig.Lock()
 	defer c.SysConfig.Unlock()
 
@@ -191,9 +193,11 @@ func (c *Config) loadConfiguration(rawConfig []byte) {
 	if err := json.Unmarshal(rawConfig, &c.SysConfig); err != nil {
 		// we only log the parser error, giving the user a chance to write a valid config
 		log.Error("Error parsing firewall configuration %s: %s", c.file, err)
-		return
+		return err
 	}
 	log.Info("fw configuration loaded")
+
+	return nil
 }
 
 // SaveConfiguration saves configuration to disk.
@@ -205,8 +209,6 @@ func (c *Config) SaveConfiguration(rawConfig string) error {
 		return err
 	}
 
-	c.loadConfiguration([]byte(rawConfig))
-
 	if err = os.Chmod(c.file, 0600); err != nil {
 		log.Warning("unable to set system-fw.json permissions: %s", err)
 	}
