@@ -50,6 +50,7 @@ import (
 	"github.com/evilsocket/opensnitch/daemon/rule"
 	"github.com/evilsocket/opensnitch/daemon/statistics"
 	"github.com/evilsocket/opensnitch/daemon/ui"
+	"github.com/evilsocket/opensnitch/daemon/ui/config"
 	"github.com/evilsocket/opensnitch/daemon/ui/protocol"
 )
 
@@ -60,7 +61,9 @@ var (
 	logFile           = ""
 	logUTC            = true
 	logMicro          = false
-	rulesPath         = "rules"
+	rulesPath         = "/etc/opensnitchd/rules/"
+	configFile        = "/etc/opensnitchd/default-config.json"
+	ebpfModPath       = "" // /usr/lib/opensnitchd/ebpf
 	noLiveReload      = false
 	queueNum          = 0
 	repeatQueueNum    int //will be set later to queueNum + 1
@@ -97,11 +100,13 @@ func init() {
 
 	flag.StringVar(&procmonMethod, "process-monitor-method", procmonMethod, "How to search for processes path. Options: ftrace, audit (experimental), ebpf (experimental), proc (default)")
 	flag.StringVar(&uiSocket, "ui-socket", uiSocket, "Path the UI gRPC service listener (https://github.com/grpc/grpc/blob/master/doc/naming.md).")
-	flag.StringVar(&rulesPath, "rules-path", rulesPath, "Path to load JSON rules from.")
 	flag.IntVar(&queueNum, "queue-num", queueNum, "Netfilter queue number.")
 	flag.IntVar(&workers, "workers", workers, "Number of concurrent workers.")
 	flag.BoolVar(&noLiveReload, "no-live-reload", debug, "Disable rules live reloading.")
 
+	flag.StringVar(&rulesPath, "rules-path", rulesPath, "Path to load JSON rules from.")
+	flag.StringVar(&configFile, "config-file", configFile, "Path to the daemon configuration file.")
+	//flag.StringVar(&ebpfModPath, "ebpf-modules-path", ebpfModPath, "Path to the directory with the eBPF modules.")
 	flag.StringVar(&logFile, "log-file", logFile, "Write logs to this file instead of the standard output.")
 	flag.BoolVar(&logUTC, "log-utc", logUTC, "Write logs output with UTC timezone (enabled by default).")
 	flag.BoolVar(&logMicro, "log-micro", logMicro, "Write logs output with microsecond timestamp (disabled by default).")
@@ -114,6 +119,27 @@ func init() {
 	flag.StringVar(&memProfile, "mem-profile", memProfile, "Write memory profile to this file.")
 }
 
+// Load configuration file from disk, by default from /etc/opensnitchd/default-config.json,
+// or from the path specified by configFile.
+// This configuration will be loaded again by uiClient(), in order to monitor it for changes.
+func loadDiskConfiguration() (*config.Config, error) {
+	if configFile == "" {
+		return nil, fmt.Errorf("Configuration file cannot be empty")
+	}
+
+	raw, err := config.Load(configFile)
+	if err != nil || len(raw) == 0 {
+		return nil, fmt.Errorf("Error loading configuration %s: %s", configFile, err)
+	}
+	clientConfig, err := config.Parse(raw)
+	if err != nil {
+		return nil, fmt.Errorf("Error parsing configuration %s: %s", configFile, err)
+	}
+
+	log.Info("Loading configuration file %s ...", configFile)
+	return &clientConfig, nil
+}
+
 func overwriteLogging() bool {
 	return debug || warning || important || errorlog || logFile != "" || logMicro
 }
@@ -318,7 +344,7 @@ func onPacket(packet netfilter.Packet) {
 	// Parse the connection state
 	con := conman.Parse(packet, uiClient.InterceptUnknown())
 	if con == nil {
-		applyDefaultAction(&packet)
+		applyDefaultAction(&packet, nil)
 		return
 	}
 	// accept our own connections
@@ -338,12 +364,15 @@ func onPacket(packet netfilter.Packet) {
 	stats.OnConnectionEvent(con, r, r == nil)
 }
 
-func applyDefaultAction(packet *netfilter.Packet) {
+func applyDefaultAction(packet *netfilter.Packet, con *conman.Connection) {
 	if uiClient.DefaultAction() == rule.Allow {
 		packet.SetVerdictAndMark(netfilter.NF_ACCEPT, packet.Mark)
-	} else {
-		packet.SetVerdict(netfilter.NF_DROP)
+		return
 	}
+	if uiClient.DefaultAction() == rule.Reject && con != nil {
+		netlink.KillSocket(con.Protocol, con.SrcIP, con.SrcPort, con.DstIP, con.DstPort)
+	}
+	packet.SetVerdict(netfilter.NF_DROP)
 }
 
 func acceptOrDeny(packet *netfilter.Packet, con *conman.Connection) *rule.Rule {
@@ -356,7 +385,7 @@ func acceptOrDeny(packet *netfilter.Packet, con *conman.Connection) *rule.Rule {
 		// send a request to the UI client if
 		// 1) connected and running and 2) we are not already asking
 		if uiClient.Connected() == false || uiClient.GetIsAsking() == true {
-			applyDefaultAction(packet)
+			applyDefaultAction(packet, con)
 			log.Debug("UI is not running or busy, connected: %v, running: %v", uiClient.Connected(), uiClient.GetIsAsking())
 			return nil
 		}
@@ -398,7 +427,7 @@ func acceptOrDeny(packet *netfilter.Packet, con *conman.Connection) *rule.Rule {
 		r = uiClient.Ask(con)
 		if r == nil {
 			log.Error("Invalid rule received, applying default action")
-			applyDefaultAction(packet)
+			applyDefaultAction(packet, con)
 			return nil
 		}
 		ok := false
@@ -440,7 +469,7 @@ func acceptOrDeny(packet *netfilter.Packet, con *conman.Connection) *rule.Rule {
 	}
 
 	if r.Enabled == false {
-		applyDefaultAction(packet)
+		applyDefaultAction(packet, con)
 		ruleName := log.Green(r.Name)
 		log.Info("DISABLED (%s) %s %s -> %s:%d (%s)", uiClient.DefaultAction(), log.Bold(log.Green("✔")), log.Bold(con.Process.Path), log.Bold(con.To()), con.DstPort, ruleName)
 
@@ -482,6 +511,17 @@ func main() {
 
 	log.Important("Starting %s v%s", core.Name, core.Version)
 
+	cfg, err := loadDiskConfiguration()
+	if err != nil {
+		log.Fatal("%s", err)
+	}
+	if err == nil && cfg.Rules.Path != "" {
+		rulesPath = cfg.Rules.Path
+	}
+	if rulesPath == "" {
+		log.Fatal("rules path cannot be empty")
+	}
+
 	rulesPath, err := core.ExpandPath(rulesPath)
 	if err != nil {
 		log.Fatal("Error accessing rules path (does it exist?): %s", err)
@@ -490,14 +530,15 @@ func main() {
 	setupSignals()
 
 	log.Info("Loading rules from %s ...", rulesPath)
-	if rules, err = rule.NewLoader(!noLiveReload); err != nil {
+	rules, err = rule.NewLoader(!noLiveReload)
+	if err != nil {
 		log.Fatal("%s", err)
 	} else if err = rules.Load(rulesPath); err != nil {
 		log.Fatal("%s", err)
 	}
 	stats = statistics.New(rules)
 	loggerMgr = loggers.NewLoggerManager()
-	uiClient = ui.NewClient(uiSocket, stats, rules, loggerMgr)
+	uiClient = ui.NewClient(uiSocket, configFile, stats, rules, loggerMgr)
 
 	// prepare the queue
 	setupWorkers()
@@ -535,15 +576,15 @@ func main() {
 	// overwrite monitor method from configuration if the user has passed
 	// the option via command line.
 	if procmonMethod != "" {
-		if err := monitor.ReconfigureMonitorMethod(procmonMethod); err != nil {
+		if err := monitor.ReconfigureMonitorMethod(procmonMethod, cfg.Ebpf.ModulesPath); err != nil {
 			msg := fmt.Sprintf("Unable to set process monitor method via parameter: %v", err)
 			uiClient.SendWarningAlert(msg)
 			log.Warning(msg)
 		}
 	}
 
-	go func(uiClient *ui.Client) {
-		if err := dns.ListenerEbpf(); err != nil {
+	go func(uiClient *ui.Client, ebpfPath string) {
+		if err := dns.ListenerEbpf(ebpfPath); err != nil {
 			msg := fmt.Sprintf("EBPF-DNS: Unable to attach ebpf listener: %s", err)
 			log.Warning(msg)
 			// don't display an alert, since this module is not critical
@@ -555,7 +596,7 @@ func main() {
 				msg)
 
 		}
-	}(uiClient)
+	}(uiClient, cfg.Ebpf.ModulesPath)
 
 	initSystemdResolvedMonitor()
 
