@@ -7,6 +7,8 @@ import (
 	"syscall"
 
 	"github.com/evilsocket/opensnitch/daemon/log"
+	"github.com/vishvananda/netlink"
+	"golang.org/x/sys/unix"
 )
 
 // GetSocketInfo asks the kernel via netlink for a given connection.
@@ -151,9 +153,78 @@ func KillSocket(proto string, srcIP net.IP, srcPort uint, dstIP net.IP, dstPort
 
 	if sockList, err := SocketGet(family, ipproto, uint16(srcPort), uint16(dstPort), srcIP, dstIP); err == nil {
 		for _, s := range sockList {
-			if err := socketKill(family, ipproto, s.ID); err != nil {
+			if err := SocketKill(family, ipproto, s.ID); err != nil {
 				log.Debug("Unable to kill socket: %d, %d, %v", srcPort, dstPort, err)
 			}
 		}
 	}
 }
+
+// KillSockets kills all sockets given a family and a protocol.
+// Be careful if you don't exclude local sockets, many local servers may misbehave,
+// entering in an infinite loop.
+func KillSockets(fam, proto uint8, excludeLocal bool) error {
+	sockListTCP, err := SocketsDump(fam, proto)
+	if err != nil {
+		return fmt.Errorf("eBPF could not dump TCP (%d/%d) sockets via netlink: %v", fam, proto, err)
+	}
+
+	for _, sock := range sockListTCP {
+		if excludeLocal && (isPrivate(sock.ID.Destination) ||
+			sock.ID.Source.IsUnspecified() ||
+			sock.ID.Destination.IsUnspecified()) {
+			continue
+		}
+		if err := SocketKill(fam, proto, sock.ID); err != nil {
+			log.Debug("Unable to kill socket (%+v): %s", sock.ID, err)
+		}
+	}
+
+	return nil
+}
+
+// KillAllSockets kills the sockets for the given families and protocols.
+func KillAllSockets() {
+	type opts struct {
+		fam   uint8
+		proto uint8
+	}
+	optList := []opts{
+		// add families and protos as wish
+		{unix.AF_INET, uint8(syscall.IPPROTO_TCP)},
+		{unix.AF_INET6, uint8(syscall.IPPROTO_TCP)},
+		{unix.AF_INET, uint8(syscall.IPPROTO_UDP)},
+		{unix.AF_INET6, uint8(syscall.IPPROTO_UDP)},
+		{unix.AF_INET, uint8(syscall.IPPROTO_SCTP)},
+		{unix.AF_INET6, uint8(syscall.IPPROTO_SCTP)},
+	}
+	for _, opt := range optList {
+		KillSockets(opt.fam, opt.proto, true)
+	}
+
+}
+
+// FlushConnections flushes conntrack as soon as netfilter rule is set.
+// This ensures that already-established connections will go to netfilter queue.
+func FlushConnections() {
+	if err := netlink.ConntrackTableFlush(netlink.ConntrackTable); err != nil {
+		log.Error("error flushing ConntrackTable %s", err)
+	}
+	if err := netlink.ConntrackTableFlush(netlink.ConntrackExpectTable); err != nil {
+		log.Error("error flusing ConntrackExpectTable %s", err)
+	}
+
+	// Force established connections to reestablish again.
+	KillAllSockets()
+}
+
+// SocketsAreEqual compares 2 different sockets to see if they match.
+func SocketsAreEqual(aSocket, bSocket *Socket) bool {
+	return ((*aSocket).INode == (*bSocket).INode &&
+		//inodes are unique enough, so the matches below will never have to be checked
+		(*aSocket).ID.SourcePort == (*bSocket).ID.SourcePort &&
+		(*aSocket).ID.Source.Equal((*bSocket).ID.Source) &&
+		(*aSocket).ID.Destination.Equal((*bSocket).ID.Destination) &&
+		(*aSocket).ID.DestinationPort == (*bSocket).ID.DestinationPort &&
+		(*aSocket).UID == (*bSocket).UID)
+}
