@@ -4,8 +4,10 @@ import (
 	"encoding/binary"
 	"fmt"
 	"net"
+	"strconv"
 	"unsafe"
 
+	"github.com/evilsocket/opensnitch/daemon/core"
 	"github.com/evilsocket/opensnitch/daemon/log"
 	daemonNetlink "github.com/evilsocket/opensnitch/daemon/netlink"
 	"github.com/evilsocket/opensnitch/daemon/procmon"
@@ -100,7 +102,12 @@ func getPidFromEbpf(proto string, srcPort uint, srcIP net.IP, dstIP net.IP, dstP
 	}
 	hostByteOrder.PutUint16(key[0:2], uint16(srcPort))
 
-	k := fmt.Sprint(proto, srcPort, srcIP.String(), dstIP.String(), dstPort)
+	k := core.ConcatStrings(
+		proto,
+		strconv.FormatUint(uint64(srcPort), 10),
+		srcIP.String(),
+		dstIP.String(),
+		strconv.FormatUint(uint64(dstPort), 10))
 	if cacheItem, isInCache := ebpfCache.isInCache(k); isInCache {
 		// should we re-read the info?
 		// environ vars might have changed
@@ -165,7 +172,7 @@ func findConnProcess(value *networkEventT, connKey string) (proc *procmon.Proces
 	proc.UID = int(value.UID)
 
 	err := proc.ReadPath()
-	if ev, found := execEvents.isInStore(value.Pid); found {
+	if ev, found := execEvents.isInStore(uint32(value.Pid)); found {
 		// use socket's UID. See above why ^
 		ev.Proc.UID = proc.UID
 		ev.Proc.ReadCmdline()
@@ -186,8 +193,8 @@ func findConnProcess(value *networkEventT, connKey string) (proc *procmon.Proces
 		log.Debug("[ebpf conn] not in cache, NOR in execEvents: %s, %d -> %s", connKey, proc.ID, proc.Path)
 		// We'll end here if the events module has not been loaded, or if the process is not in cache.
 		proc.GetInfo()
-		execEvents.add(value.Pid,
-			*NewExecEvent(value.Pid, 0, value.UID, proc.Path, value.Comm),
+		execEvents.add(uint32(value.Pid),
+			*NewExecEvent(uint32(value.Pid), 0, uint32(value.UID), proc.Path, value.Comm),
 			*proc)
 	}
 
@@ -220,10 +227,6 @@ func findInAlreadyEstablishedTCP(proto string, srcPort uint, srcIP net.IP, dstIP
 func findAddressInLocalAddresses(addr net.IP) bool {
 	lock.Lock()
 	defer lock.Unlock()
-	for _, a := range localAddresses {
-		if addr.String() == a.String() {
-			return true
-		}
-	}
-	return false
+	_, found := localAddresses[addr.String()]
+	return found
 }
