@@ -1,6 +1,7 @@
 package rule
 
 import (
+	"fmt"
 	"io"
 	"math/rand"
 	"os"
@@ -39,22 +40,22 @@ func TestRuleLoader(t *testing.T) {
 	if err = l.Load("testdata/"); err != nil {
 		t.Error("Error loading test rules: ", err)
 	}
-
-	testNumRules(t, l, 2)
+	// we expect 6 valid rules (2 invalid), loaded from testdata/
+	testNumRules(t, l, 6)
 
 	if err = l.Add(inMem1sRule, false); err != nil {
 		t.Error("Error adding temporary rule")
 	}
-	testNumRules(t, l, 3)
+	testNumRules(t, l, 7)
 
 	// test auto deletion of temporary rule
 	time.Sleep(time.Second * 2)
-	testNumRules(t, l, 2)
+	testNumRules(t, l, 6)
 
 	if err = l.Add(inMemUntilRestartRule, false); err != nil {
 		t.Error("Error adding temporary rule (2)")
 	}
-	testNumRules(t, l, 3)
+	testNumRules(t, l, 7)
 	testRulesOrder(t, l)
 	testSortRules(t, l)
 	testFindMatch(t, l)
@@ -95,6 +96,57 @@ func TestRuleLoaderInvalidRegexp(t *testing.T) {
 	})
 }
 
+// Test rules of type operator.list. There're these scenarios:
+// - Enabled rules:
+//    * operator Data field is ignored if it contains the list of operators as json string.
+//    * the operarots list is expanded as json objecs under "list": []
+// For new rules (> v1.6.3), Data field will be empty.
+//
+// - Disabled rules
+//    * (old) the Data field contains the list of operators as json string, and the list of operarots is empty.
+//    * Data field empty, and the list of operators expanded.
+// In all cases the list of operators must be loaded.
+func TestRuleLoaderList(t *testing.T) {
+	l, err := NewLoader(true)
+	if err != nil {
+		t.Fail()
+	}
+
+	testRules := map[string]string{
+		"rule-with-operator-list":                          "testdata/rule-operator-list.json",
+		"rule-disabled-with-operators-list-as-json-string": "testdata/rule-disabled-operator-list.json",
+		"rule-disabled-with-operators-list-expanded":       "testdata/rule-disabled-operator-list-expanded.json",
+		"rule-with-operator-list-data-empty":               "testdata/rule-operator-list-data-empty.json",
+	}
+
+	for name, path := range testRules {
+		t.Run(fmt.Sprint("loadRule() ", path), func(t *testing.T) {
+			if err := l.loadRule(path); err != nil {
+				t.Error(fmt.Sprint("loadRule() ", path, " error:"), err)
+			}
+			t.Log("Test: List rule:", name, path)
+			r, found := l.rules[name]
+			if !found {
+				t.Error(fmt.Sprint("loadRule() ", path, " not in the list:"), l.rules)
+			}
+			// Starting from > v1.6.3, after loading a rule of type List, the field Operator.Data is emptied, if the Data contained the list of operators as json.
+			if len(r.Operator.List) != 2 {
+				t.Error(fmt.Sprint("loadRule() ", path, " operator List not loaded:"), r)
+			}
+			if r.Operator.List[0].Type != Simple ||
+				r.Operator.List[0].Operand != OpProcessPath ||
+				r.Operator.List[0].Data != "/usr/bin/telnet" {
+				t.Error(fmt.Sprint("loadRule() ", path, " operator List 0 not loaded:"), r)
+			}
+			if r.Operator.List[1].Type != Simple ||
+				r.Operator.List[1].Operand != OpDstPort ||
+				r.Operator.List[1].Data != "53" {
+				t.Error(fmt.Sprint("loadRule() ", path, " operator List 1 not loaded:"), r)
+			}
+		})
+	}
+}
+
 func TestLiveReload(t *testing.T) {
 	t.Parallel()
 	t.Log("Test rules loader with live reload")
