@@ -33,22 +33,24 @@ const (
 // The fields match the ones saved as json to disk.
 // If a .json rule file is modified on disk, it's reloaded automatically.
 type Rule struct {
-	Created     time.Time `json:"created"`
-	Updated     time.Time `json:"updated"`
-	Name        string    `json:"name"`
-	Description string    `json:"description"`
-	Enabled     bool      `json:"enabled"`
-	Precedence  bool      `json:"precedence"`
-	Nolog       bool      `json:"nolog"`
-	Action      Action    `json:"action"`
-	Duration    Duration  `json:"duration"`
-	Operator    Operator  `json:"operator"`
+	// Save date fields as string, to avoid issues marshalling Time (#1140).
+	Created string `json:"created"`
+	Updated string `json:"updated"`
+
+	Name        string   `json:"name"`
+	Description string   `json:"description"`
+	Action      Action   `json:"action"`
+	Duration    Duration `json:"duration"`
+	Operator    Operator `json:"operator"`
+	Enabled     bool     `json:"enabled"`
+	Precedence  bool     `json:"precedence"`
+	Nolog       bool     `json:"nolog"`
 }
 
 // Create creates a new rule object with the specified parameters.
 func Create(name, description string, enabled, precedence, nolog bool, action Action, duration Duration, op *Operator) *Rule {
 	return &Rule{
-		Created:     time.Now(),
+		Created:     time.Now().Format(time.RFC3339),
 		Enabled:     enabled,
 		Precedence:  precedence,
 		Nolog:       nolog,
@@ -61,7 +63,11 @@ func Create(name, description string, enabled, precedence, nolog bool, action Ac
 }
 
 func (r *Rule) String() string {
-	return fmt.Sprintf("%s: if(%s){ %s %s }", r.Name, r.Operator.String(), r.Action, r.Duration)
+	enabled := "Disabled"
+	if r.Enabled {
+		enabled = "Enabled"
+	}
+	return fmt.Sprintf("[%s] %s: if(%s){ %s %s }", enabled, r.Name, r.Operator.String(), r.Action, r.Duration)
 }
 
 // Match performs on a connection the checks a Rule has, to determine if it
@@ -88,7 +94,7 @@ func Deserialize(reply *protocol.Rule) (*Rule, error) {
 		return nil, err
 	}
 
-	return Create(
+	newRule := Create(
 		reply.Name,
 		reply.Description,
 		reply.Enabled,
@@ -97,7 +103,25 @@ func Deserialize(reply *protocol.Rule) (*Rule, error) {
 		Action(reply.Action),
 		Duration(reply.Duration),
 		operator,
-	), nil
+	)
+
+	if Type(reply.Operator.Type) == List {
+		newRule.Operator.Data = ""
+		reply.Operator.Data = ""
+		for i := 0; i < len(reply.Operator.List); i++ {
+			newRule.Operator.List = append(
+				newRule.Operator.List,
+				Operator{
+					Type:      Type(reply.Operator.List[i].Type),
+					Sensitive: Sensitive(reply.Operator.List[i].Sensitive),
+					Operand:   Operand(reply.Operator.List[i].Operand),
+					Data:      string(reply.Operator.List[i].Data),
+				},
+			)
+		}
+	}
+
+	return newRule, nil
 }
 
 // Serialize translates a Rule to the protocol object
@@ -105,7 +129,16 @@ func (r *Rule) Serialize() *protocol.Rule {
 	if r == nil {
 		return nil
 	}
-	return &protocol.Rule{
+
+	created, err := time.Parse(time.RFC3339, r.Created)
+	if err != nil {
+		log.Warning("Error parsing rule Created date (it should be in RFC3339 format): %s  (%s)", err, string(r.Name))
+		log.Warning("using current time instead: %s", created)
+		created = time.Now()
+	}
+
+	protoRule := &protocol.Rule{
+		Created:     created.Unix(),
 		Name:        string(r.Name),
 		Description: string(r.Description),
 		Enabled:     bool(r.Enabled),
@@ -120,4 +153,18 @@ func (r *Rule) Serialize() *protocol.Rule {
 			Data:      string(r.Operator.Data),
 		},
 	}
+	if r.Operator.Type == List {
+		r.Operator.Data = ""
+		for i := 0; i < len(r.Operator.List); i++ {
+			protoRule.Operator.List = append(protoRule.Operator.List,
+				&protocol.Operator{
+					Type:      string(r.Operator.List[i].Type),
+					Sensitive: bool(r.Operator.List[i].Sensitive),
+					Operand:   string(r.Operator.List[i].Operand),
+					Data:      string(r.Operator.List[i].Data),
+				})
+		}
+	}
+
+	return protoRule
 }
