@@ -1,7 +1,7 @@
 package statistics
 
 import (
-	"fmt"
+	"strconv"
 	"sync"
 	"time"
 
@@ -29,32 +29,30 @@ type conEvent struct {
 // Statistics holds the connections and statistics the daemon intercepts.
 // The connections are stored in the Events slice.
 type Statistics struct {
-	sync.RWMutex
-
+	logger       *loggers.LoggerManager
+	rules        *rule.Loader
 	Started      time.Time
+	ByExecutable map[string]uint64
+	ByPort       map[string]uint64
+	ByProto      map[string]uint64
+	ByAddress    map[string]uint64
+	ByHost       map[string]uint64
+	jobs         chan conEvent
+	ByUID        map[string]uint64
+	Events       []*Event
+	Dropped      int
+	// max number of events to keep in the buffer
+	maxEvents int
+	// max number of entries for each By* map
+	maxStats     int
 	DNSResponses int
 	Connections  int
 	Ignored      int
 	Accepted     int
-	Dropped      int
 	RuleHits     int
 	RuleMisses   int
-	Events       []*Event
-	ByProto      map[string]uint64
-	ByAddress    map[string]uint64
-	ByHost       map[string]uint64
-	ByPort       map[string]uint64
-	ByUID        map[string]uint64
-	ByExecutable map[string]uint64
 
-	rules *rule.Loader
-	jobs  chan conEvent
-	// max number of events to keep in the buffer
-	maxEvents int
-	// max number of entries for each By* map
-	maxStats int
-
-	logger *loggers.LoggerManager
+	sync.RWMutex
 }
 
 // New returns a new Statistics object and initializes the go routines to update the stats.
@@ -198,8 +196,8 @@ func (s *Statistics) onConnection(con *conman.Connection, match *rule.Rule, wasM
 	if con.DstHost != "" {
 		s.incMap(&s.ByHost, con.DstHost)
 	}
-	s.incMap(&s.ByPort, fmt.Sprintf("%d", con.DstPort))
-	s.incMap(&s.ByUID, fmt.Sprintf("%d", con.Entry.UserId))
+	s.incMap(&s.ByPort, strconv.FormatUint(uint64(con.DstPort), 10))
+	s.incMap(&s.ByUID, strconv.Itoa(con.Entry.UserId))
 	s.incMap(&s.ByExecutable, con.Process.Path)
 
 	// if we reached the limit, shift everything back
