@@ -12,6 +12,8 @@ import (
 	"github.com/evilsocket/opensnitch/daemon/log/loggers"
 	"github.com/evilsocket/opensnitch/daemon/rule"
 	"github.com/evilsocket/opensnitch/daemon/statistics"
+	"github.com/evilsocket/opensnitch/daemon/ui/auth"
+	"github.com/evilsocket/opensnitch/daemon/ui/config"
 	"github.com/evilsocket/opensnitch/daemon/ui/protocol"
 
 	"github.com/fsnotify/fsnotify"
@@ -28,56 +30,37 @@ var (
 	// While the GUI is connected, deny by default everything until the user takes an action.
 	clientConnectedRule = rule.Create("ui.client.connected", "", true, false, false, rule.Deny, rule.Once, dummyOperator)
 	clientErrorRule     = rule.Create("ui.client.error", "", true, false, false, rule.Allow, rule.Once, dummyOperator)
-	config              Config
+	clientConfig        config.Config
 
 	maxQueuedAlerts = 1024
 )
 
-type serverConfig struct {
-	Address string                 `json:"Address"`
-	LogFile string                 `json:"LogFile"`
-	Loggers []loggers.LoggerConfig `json:"Loggers"`
-}
-
-// Config holds the values loaded from configFile
-type Config struct {
-	sync.RWMutex
-	Server            serverConfig           `json:"Server"`
-	DefaultAction     string                 `json:"DefaultAction"`
-	DefaultDuration   string                 `json:"DefaultDuration"`
-	InterceptUnknown  bool                   `json:"InterceptUnknown"`
-	ProcMonitorMethod string                 `json:"ProcMonitorMethod"`
-	LogLevel          *uint32                `json:"LogLevel"`
-	LogUTC            bool                   `json:"LogUTC"`
-	LogMicro          bool                   `json:"LogMicro"`
-	Firewall          string                 `json:"Firewall"`
-	Stats             statistics.StatsConfig `json:"Stats"`
-}
-
 // Client holds the connection information of a client.
 type Client struct {
-	sync.RWMutex
-	clientCtx    context.Context
-	clientCancel context.CancelFunc
-
-	stats         *statistics.Statistics
-	rules         *rule.Loader
-	socketPath    string
-	isUnixSocket  bool
-	con           *grpc.ClientConn
-	client        protocol.UIClient
-	configWatcher *fsnotify.Watcher
-
+	rules               *rule.Loader
+	stats               *statistics.Statistics
+	con                 *grpc.ClientConn
+	configWatcher       *fsnotify.Watcher
+	client              protocol.UIClient
+	clientCtx           context.Context
+	clientCancel        context.CancelFunc
+	streamNotifications protocol.UI_NotificationsClient
 	isConnected         chan bool
 	alertsChan          chan protocol.Alert
-	streamNotifications protocol.UI_NotificationsClient
-
+	socketPath          string
+	unixSockPrefix      string
 	//isAsking is set to true if the client is awaiting a decision from the GUI
-	isAsking bool
+	isAsking     bool
+	isUnixSocket bool
+
+	sync.RWMutex
 }
 
 // NewClient creates and configures a new client.
-func NewClient(socketPath string, stats *statistics.Statistics, rules *rule.Loader, loggers *loggers.LoggerManager) *Client {
+func NewClient(socketPath, localConfigFile string, stats *statistics.Statistics, rules *rule.Loader, loggers *loggers.LoggerManager) *Client {
+	if localConfigFile != "" {
+		configFile = localConfigFile
+	}
 	c := &Client{
 		stats:        stats,
 		rules:        rules,
@@ -98,8 +81,8 @@ func NewClient(socketPath string, stats *statistics.Statistics, rules *rule.Load
 	if socketPath != "" {
 		c.setSocketPath(c.getSocketPath(socketPath))
 	}
-	loggers.Load(config.Server.Loggers, config.Stats.Workers)
-	stats.SetLimits(config.Stats)
+	loggers.Load(clientConfig.Server.Loggers, clientConfig.Stats.Workers)
+	stats.SetLimits(clientConfig.Stats)
 	stats.SetLoggers(loggers)
 
 	return c
@@ -118,26 +101,26 @@ func (c *Client) Close() {
 // ProcMonitorMethod returns the monitor method configured.
 // If it's not present in the config file, it'll return an empty string.
 func (c *Client) ProcMonitorMethod() string {
-	config.RLock()
-	defer config.RUnlock()
-	return config.ProcMonitorMethod
+	clientConfig.RLock()
+	defer clientConfig.RUnlock()
+	return clientConfig.ProcMonitorMethod
 }
 
 // InterceptUnknown returns
 func (c *Client) InterceptUnknown() bool {
-	config.RLock()
-	defer config.RUnlock()
-	return config.InterceptUnknown
+	clientConfig.RLock()
+	defer clientConfig.RUnlock()
+	return clientConfig.InterceptUnknown
 }
 
 // GetFirewallType returns the firewall to use
 func (c *Client) GetFirewallType() string {
-	config.RLock()
-	defer config.RUnlock()
-	if config.Firewall == "" {
+	clientConfig.RLock()
+	defer clientConfig.RUnlock()
+	if clientConfig.Firewall == "" {
 		return iptables.Name
 	}
-	return config.Firewall
+	return clientConfig.Firewall
 }
 
 // DefaultAction returns the default configured action for
@@ -172,14 +155,14 @@ func (c *Client) Connected() bool {
 	return true
 }
 
-//GetIsAsking returns the isAsking flag
+// GetIsAsking returns the isAsking flag
 func (c *Client) GetIsAsking() bool {
 	c.RLock()
 	defer c.RUnlock()
 	return c.isAsking
 }
 
-//SetIsAsking sets the isAsking flag
+// SetIsAsking sets the isAsking flag
 func (c *Client) SetIsAsking(flag bool) {
 	c.Lock()
 	defer c.Unlock()
@@ -250,6 +233,7 @@ func (c *Client) connect() (err error) {
 	}
 
 	if err := c.openSocket(); err != nil {
+		log.Debug("connect() %s", err)
 		c.disconnect()
 		return err
 	}
@@ -264,10 +248,14 @@ func (c *Client) openSocket() (err error) {
 	c.Lock()
 	defer c.Unlock()
 
+	dialOption, err := auth.New(&clientConfig)
+	if err != nil {
+		return fmt.Errorf("Invalid client auth options: %s", err)
+	}
 	if c.isUnixSocket {
-		c.con, err = grpc.Dial(c.socketPath, grpc.WithInsecure(),
+		c.con, err = grpc.Dial(c.socketPath, dialOption,
 			grpc.WithDialer(func(addr string, timeout time.Duration) (net.Conn, error) {
-				return net.DialTimeout("unix", addr, timeout)
+				return net.DialTimeout(c.unixSockPrefix, addr, timeout)
 			}))
 	} else {
 		// https://pkg.go.dev/google.golang.org/grpc/keepalive#ClientParameters
@@ -280,7 +268,7 @@ func (c *Client) openSocket() (err error) {
 			PermitWithoutStream: true,
 		}
 
-		c.con, err = grpc.Dial(c.socketPath, grpc.WithInsecure(), grpc.WithKeepaliveParams(kacp))
+		c.con, err = grpc.Dial(c.socketPath, dialOption, grpc.WithKeepaliveParams(kacp))
 	}
 
 	return err
