@@ -3,8 +3,8 @@
 app_name="opensnitch"
 langs_dir="./locales"
 lrelease="lrelease"
-if ! command -v $lrelease; then
-    # fedora
+if ! command -v lrelease >/dev/null; then
+    # on fedora
     lrelease="lrelease-qt5"
 fi
 
