@@ -1,6 +1,5 @@
 from threading import Lock
 import configparser
-import pyinotify
 import threading
 import glob
 import os
@@ -8,6 +7,13 @@ import re
 import shutil
 import locale
 
+is_pyinotify_available = True
+try:
+    import pyinotify
+except Exception as e:
+    is_pyinotify_available = False
+    print("Error importing pyinotify:", e)
+
 DESKTOP_PATHS = tuple([
     os.path.join(d, 'applications')
     for d in os.getenv('XDG_DATA_DIRS', '/usr/share/').split(':')
@@ -37,7 +43,8 @@ class LinuxDesktopParser(threading.Thread):
             for desktop_file in glob.glob(os.path.join(desktop_path, '*.desktop')):
                 self._parse_desktop_file(desktop_file)
 
-        self.start()
+        if is_pyinotify_available:
+            self.start()
 
     def get_locale(self):
         try:
@@ -49,7 +56,7 @@ class LinuxDesktopParser(threading.Thread):
 
     def _parse_exec(self, cmd):
         try:
-            is_flatpak = re.search("^/usr/[s]*bin/flatpak.*--command=([a-zA-Z0-9-_\/\.\+]+)", cmd)
+            is_flatpak = re.search(r'^/usr/[s]*bin/flatpak.*--command=([a-zA-Z0-9-_\/\.\+]+)', cmd)
             if is_flatpak:
                 return is_flatpak.group(1)
 
