@@ -9,10 +9,11 @@ from PyQt5.QtCore import QCoreApplication as QC
 from opensnitch.config import Config
 from opensnitch.nodes import Nodes
 from opensnitch.utils import NetworkServices, NetworkInterfaces, QuickHelp, Icons, Utils
-from opensnitch import ui_pb2
 import opensnitch.firewall as Fw
 from opensnitch.firewall.utils import Utils as FwUtils
 
+import opensnitch.proto as proto
+ui_pb2, ui_pb2_grpc = proto.import_()
 
 DIALOG_UI_PATH = "%s/../res/firewall_rule.ui" % os.path.dirname(sys.modules[__name__].__file__)
 class FwRuleDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
@@ -144,14 +145,16 @@ Supported formats:
             },
             self.STATM_IIFNAME: {
                 'name': Fw.Statements.IIFNAME.value,
-                'tooltip': QC.translate("firewall", "Match input interface. Regular expressions not allowed."),
+                'tooltip': QC.translate("firewall", """Match input interface. Regular expressions not allowed.
+Use * to match multiple interfaces."""),
                 'keys': [
                     {'key': "", 'values': []}
                 ]
             },
             self.STATM_OIFNAME: {
                 'name': Fw.Statements.OIFNAME.value,
-                'tooltip': QC.translate("firewall", "Match output interface. Regular expressions not allowed."),
+                'tooltip': QC.translate("firewall", """Match output interface. Regular expressions not allowed.
+Use * to match multiple interfaces."""),
                 'keys': [
                     {'key': "", 'values': []}
                 ]
@@ -332,7 +335,7 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
         self.cmdDelete.clicked.connect(self._cb_delete_clicked)
         self.helpButton.clicked.connect(self._cb_help_button_clicked)
         self.comboVerdict.currentIndexChanged.connect(self._cb_verdict_changed)
-        self.comboDirection.currentIndexChanged.connect(self._cb_direction_changed)
+        self.lineVerdictParms.textChanged.connect(self._cb_verdict_parms_changed)
         self.checkEnable.toggled.connect(self._cb_check_enable_toggled)
         self.lineDescription.textChanged.connect(self._cb_description_changed)
 
@@ -350,12 +353,12 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
 
         # -----------------------------------------------------------
 
-        saveIcon = Icons.new("document-save")
-        closeIcon = Icons.new("window-close")
-        delIcon = Icons.new("edit-delete")
-        addIcon = Icons.new("list-add")
-        remIcon = Icons.new("list-remove")
-        helpIcon = Icons.new("help-browser")
+        saveIcon = Icons.new(self, "document-save")
+        closeIcon = Icons.new(self, "window-close")
+        delIcon = Icons.new(self, "edit-delete")
+        addIcon = Icons.new(self, "list-add")
+        remIcon = Icons.new(self, "list-remove")
+        helpIcon = Icons.new(self, "help-browser")
         self.cmdSave.setIcon(saveIcon)
         self.cmdDelete.setIcon(delIcon)
         self.cmdClose.setIcon(closeIcon)
@@ -380,31 +383,54 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
             return False
 
         self._load_nodes()
+        self.comboDirection.currentIndexChanged.connect(self._cb_direction_changed)
         return True
 
-
     def _close(self):
+        self.comboDirection.currentIndexChanged.disconnect(self._cb_direction_changed)
         self.hide()
 
     @QtCore.pyqtSlot(ui_pb2.NotificationReply)
     def _cb_notification_callback(self, reply):
         self._enable_buttons()
 
-        if reply.id in self._notifications_sent:
+        try:
+            if reply.id not in self._notifications_sent:
+                return
+
+            rep = self._notifications_sent[reply.id]
             if reply.code == ui_pb2.OK:
-                rep = self._notifications_sent[reply.id]
                 if 'operation' in rep and rep['operation'] == self.OP_DELETE:
                     self.tabWidget.setDisabled(True)
                     self._set_status_successful(QC.translate("firewall", "Rule deleted"))
                     self._disable_controls()
+                    del self._notifications_sent[reply.id]
                     return
 
-                self._set_status_successful(QC.translate("firewall", "Rule added"))
+                if 'operation' in rep and rep['operation'] == self.OP_SAVE:
+                    self._set_status_successful(QC.translate("firewall", "Rule saved"))
+                else:
+                    self._set_status_successful(QC.translate("firewall", "Rule added"))
 
             else:
-                self._set_status_error(QC.translate("firewall", "Error: {0}").format(reply.data))
+                # XXX: The errors returned by the nftables lib are not really descriptive.
+                # "invalid argument", "no such file or directory", without context
+                # 1st one: invalid combination of table/chain/priorities?
+                # 2nd one: does the table/chain exist?
+                errormsg = QC.translate("firewall", "Error adding rules:\n{0}".format(reply.data))
+                if 'operation' in rep and rep['operation'] == self.OP_SAVE:
+                    if 'uuid' in rep and rep['uuid'] in reply.data:
+                        errormsg = QC.translate("firewall", "Error saving rule")
+                    else:
+                        self._set_status_message(QC.translate("firewall", "Rule saved, but there're other rules with errors (REVIEW):\n{0}".format(reply.data)))
+                        return
+                self._set_status_error(errormsg)
 
-            del self._notifications_sent[reply.id]
+        except Exception as e:
+            print("[fw rule dialog exception] notif error:", e)
+        finally:
+            if reply.id in self._notifications_sent:
+                del self._notifications_sent[reply.id]
 
     @QtCore.pyqtSlot(int)
     def _cb_nodes_updated(self, total):
@@ -447,10 +473,10 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
             self._set_status_error(QC.translate("firewall", "Error updating rule"))
             return
 
-        if self.comboNodes.currentIndex() > 0:
-            self.send_notification(node_addr, node['firewall'], self.OP_DELETE)
-        else:
+        if self.comboNodes.currentIndex() == 0:
             self.send_notifications(node['firewall'], self.OP_DELETE)
+        else:
+            self.send_notification(node_addr, node['firewall'], self.OP_DELETE)
 
     def _cb_save_clicked(self):
         if len(self.statements) == 0:
@@ -464,12 +490,12 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
         self._set_status_message(QC.translate("firewall", "Adding rule, wait"))
         ok, err = self._fw.update_rule(node_addr, self.uuid, chain)
         if not ok:
-            self._set_status_error(QC.translate("firewall", "Error updating rule: {0}".format(err)))
+            self._set_status_error(QC.translate("firewall", "Error updating rule ({0}): {1}".format(node_addr, err)))
             return
 
         self._enable_buttons(False)
-        if self.comboNodes.currentIndex() > 0:
-            self.send_notification(node_addr, node['firewall'], self.OP_SAVE)
+        if self.comboNodes.currentIndex() == 0:
+            self.send_notification(node_addr, node['firewall'], self.OP_SAVE, self.uuid)
         else:
             self.send_notifications(node['firewall'], self.OP_SAVE)
 
@@ -492,8 +518,9 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
             return
         self._set_status_message(QC.translate("firewall", "Adding rule, wait"))
         self._enable_buttons(False)
-        if self.comboNodes.currentIndex() > 0:
-            self.send_notification(node_addr, node['firewall'], self.OP_NEW)
+
+        if self.comboNodes.currentIndex() == 0:
+            self.send_notification(node_addr, node['firewall'], self.OP_NEW, chain.Rules[0].UUID)
         else:
             self.send_notifications(node['firewall'], self.OP_NEW)
 
@@ -578,6 +605,10 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
         self.comboVerdictParms.setVisible(showVerdictParms)
         self._configure_verdict_parms(idx)
 
+
+    def _cb_verdict_parms_changed(self, idx):
+        self._enable_save()
+
     def _is_valid_rule(self):
         if (self.comboVerdict.currentText().lower() == Config.ACTION_REDIRECT or \
             self.comboVerdict.currentText().lower() == Config.ACTION_TPROXY or \
@@ -1134,8 +1165,9 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
 
             elif exp.Statement.Name == Fw.Statements.COUNTER.value:
                 self.statements[idx]['what'].setCurrentIndex(self.STATM_COUNTER+1)
-                if exp.Statement.Values[0].Key == Fw.ExprCounter.NAME.value:
-                    self.statements[idx]['value'].setCurrentText(exp.Statement.Values[0].Value)
+                for v in exp.Statement.Values:
+                    if v.Key == Fw.ExprCounter.NAME.value:
+                        self.statements[idx]['value'].setCurrentText(v.Value)
 
             else:
                 isNotSupported = True
@@ -1458,9 +1490,9 @@ The value must be in the format: VALUE/UNITS/TIME, for example:
 
         return True
 
-    def send_notification(self, node_addr, fw_config, op):
+    def send_notification(self, node_addr, fw_config, op, uuid):
         nid, notif = self._nodes.reload_fw(node_addr, fw_config, self._notification_callback)
-        self._notifications_sent[nid] = {'addr': node_addr, 'operation': op, 'notif': notif}
+        self._notifications_sent[nid] = {'addr': node_addr, 'operation': op, 'notif': notif, 'uuid': uuid}
 
     def send_notifications(self, fw_config, op):
         for addr in self._nodes.get_nodes():
