@@ -6,6 +6,7 @@ import os.path
 import pwd
 import json
 import ipaddress
+from datetime import datetime
 
 from PyQt5 import QtCore, QtGui, uic, QtWidgets
 from PyQt5.QtCore import QCoreApplication as QC, QEvent
@@ -16,8 +17,11 @@ from opensnitch.utils import Icons
 from opensnitch.desktop_parser import LinuxDesktopParser
 from opensnitch.config import Config
 from opensnitch.version import version
+from opensnitch.actions import Actions
+from opensnitch.rules import Rules, Rule
 
-from opensnitch import ui_pb2
+import opensnitch.proto as proto
+ui_pb2, ui_pb2_grpc = proto.import_()
 
 DIALOG_UI_PATH = "%s/../res/prompt.ui" % os.path.dirname(sys.modules[__name__].__file__)
 class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
@@ -38,6 +42,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     FIELD_DST_PORT      = "dst_port"
     FIELD_DST_NETWORK   = "dst_network"
     FIELD_DST_HOST      = "simple_host"
+    FIELD_APPIMAGE      = "appimage_path"
 
     DURATION_30s    = "30s"
     DURATION_5m     = "5m"
@@ -46,6 +51,8 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     DURATION_1h     = "1h"
     # don't translate
 
+    APPIMAGE_PREFIX = "/tmp/.mount_"
+
     # label displayed in the pop-up combo
     DURATION_session = QC.translate("popups", "until reboot")
     # label displayed in the pop-up combo
@@ -55,12 +62,15 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         QtWidgets.QDialog.__init__(self, parent, QtCore.Qt.WindowStaysOnTopHint)
         # Other interesting flags: QtCore.Qt.Tool | QtCore.Qt.BypassWindowManagerHint
         self._cfg = Config.get()
+        self._rules = Rules.instance()
+
         self.setupUi(self)
         self.setWindowIcon(appicon)
         self.installEventFilter(self)
 
-        self._width = None
-        self._height = None
+        self._width = self.width()
+        self._height = self.height()
+        self.reset_widgets()
 
         dialog_geometry = self._cfg.getSettings("promptDialog/geometry")
         if dialog_geometry == QtCore.QByteArray:
@@ -101,9 +111,9 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.checkDstPort.clicked.connect(self._button_clicked)
         self.checkUserID.clicked.connect(self._button_clicked)
 
-        self.allowIcon = Icons.new("emblem-default")
-        denyIcon = Icons.new("emblem-important")
-        rejectIcon = Icons.new("window-close")
+        self.allowIcon = Icons.new(self, "emblem-default")
+        denyIcon = Icons.new(self, "emblem-important")
+        rejectIcon = Icons.new(self, "window-close")
 
         self._default_action = self._cfg.getInt(self._cfg.DEFAULT_ACTION_KEY)
 
@@ -147,6 +157,22 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.adjust_size()
         self.move_popup()
 
+    def reset_widgets(self):
+        # Don't allow labels to grow more than the dialog's width.
+        # This can happen if the path or the binary name is too large.
+
+        self.appNameLabel.setMaximumWidth(self._width-5)
+        self.appDescriptionLabel.setMaximumWidth(self._width-5)
+        self.appPathLabel.setMaximumWidth(self._width-5)
+        self.argsLabel.setMaximumWidth(self._width-5)
+        self.messageLabel.setMaximumWidth(self._width-5)
+
+        self.appNameLabel.setText("")
+        self.appDescriptionLabel.setText("")
+        self.appPathLabel.setText("")
+        self.argsLabel.setText("")
+        self.messageLabel.setText("")
+
     def adjust_size(self):
         if self._width is None or self._height is None:
             self._width = self.width()
@@ -190,18 +216,23 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     def _button_clicked(self):
         self._stop_countdown()
 
-    def _set_elide_text(self, widget, text, max_size=132):
+    def truncate_text(self, text, max_size=64):
         if len(text) > max_size:
             text = text[:max_size] + "..."
+        return text
 
+    def _set_elide_text(self, widget, text, max_size=64):
+        text = self.truncate_text(text, max_size)
         widget.setText(text)
 
     def promptUser(self, connection, is_local, peer):
         # one at a time
         with self._lock:
             # reset state
+            self.reset_widgets()
             if self._tick_thread != None and self._tick_thread.is_alive():
                 self._tick_thread.join()
+
             self._cfg.reload()
             self._tick = int(self._cfg.getSettings(self._cfg.DEFAULT_TIMEOUT_KEY)) if self._cfg.hasKey(self._cfg.DEFAULT_TIMEOUT_KEY) else self.DEFAULT_TIMEOUT
             self._tick_thread = threading.Thread(target=self._timeout_worker)
@@ -287,6 +318,13 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.appDescriptionLabel.setVisible(False)
             self.appDescriptionLabel.setFixedHeight(0)
             self.appDescriptionLabel.setText("")
+            return
+
+        self.appDescriptionLabel.setText(
+            "".join(
+                filter(str.isprintable, self.appDescriptionLabel.text())
+            )
+        )
 
     def _set_app_path(self, app_name, app_args, con):
         # show the binary path if it's not part of the cmdline args:
@@ -305,17 +343,49 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         else:
             self.appPathLabel.setVisible(False)
             self.appPathLabel.setText("")
+            return
+
+        self.appPathLabel.setText(
+            "".join(
+                filter(str.isprintable, self.appPathLabel.text())
+            )
+        )
+        if self.appPathLabel.width() >= self._width:
+            self.appPathLabel.setText("\u200b".join(self.appPathLabel.text()))
 
     def _set_app_args(self, app_name, app_args):
         # if the app name and the args are the same, there's no need to display
         # the args label (amule for example)
         if app_name.lower() != app_args:
             self.argsLabel.setVisible(True)
-            self._set_elide_text(self.argsLabel, app_args)
+            self._set_elide_text(self.argsLabel, app_args, 256)
             self.argsLabel.setToolTip(app_args)
         else:
             self.argsLabel.setVisible(False)
             self.argsLabel.setText("")
+            return
+
+        self.argsLabel.setText(
+            "".join(
+                filter(str.isprintable, self.argsLabel.text())
+            )
+        )
+        if self.argsLabel.width() >= self._width:
+            self.argsLabel.setText("\u200b".join(self.argsLabel.text()))
+
+    def _set_default_target(self, combo, con, app_name, app_args):
+        # set appimage as default target if the process path starts with
+        # /tmp/._mount
+        if con.process_path.startswith(self.APPIMAGE_PREFIX):
+            idx = combo.findData(self.FIELD_APPIMAGE)
+            if idx != -1:
+                combo.setCurrentIndex(idx)
+                return
+
+        if int(con.process_id) > 0 and app_name != "" and app_args != "":
+            self.whatCombo.setCurrentIndex(int(self._cfg.getSettings(self._cfg.DEFAULT_TARGET_KEY)))
+        else:
+            self.whatCombo.setCurrentIndex(2)
 
     def _render_connection(self, con):
         app_name, app_icon, description, _ = self._apps_parser.get_info_by_path(con.process_path, "terminal")
@@ -384,6 +454,10 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.whatCombo.model().item(4).setEnabled(False)
 
         self.whatCombo.addItem(QC.translate("popups", "from this PID"), self.FIELD_PROC_ID)
+        #######################
+
+        if con.process_path.startswith(self.APPIMAGE_PREFIX):
+            self._add_appimage_pattern_to_combo(self.whatCombo, con)
 
         self._add_dst_networks_to_combo(self.whatCombo, con.dst_ip)
 
@@ -404,11 +478,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
         self._configure_default_duration()
 
-        if int(con.process_id) > 0:
-            self.whatCombo.setCurrentIndex(int(self._cfg.getSettings(self._cfg.DEFAULT_TARGET_KEY)))
-        else:
-            self.whatCombo.setCurrentIndex(2)
-
+        self._set_default_target(self.whatCombo, con, app_name, app_args)
 
         self.checkDstIP.setChecked(self._cfg.getBool(self._cfg.DEFAULT_POPUP_ADVANCED_DSTIP))
         self.checkDstPort.setChecked(self._cfg.getBool(self._cfg.DEFAULT_POPUP_ADVANCED_DSTPORT))
@@ -432,6 +502,17 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self._send_rule()
         e.ignore()
 
+    def _add_appimage_pattern_to_combo(self, combo, con):
+        """appimages' absolute path usually starts with /tmp/.mount_
+        """
+        appimage_bin = os.path.basename(con.process_path)
+        appimage_path = os.path.dirname(con.process_path)
+        appimage_path = appimage_path[0:len(self.APPIMAGE_PREFIX)+6]
+        combo.addItem(
+            QC.translate("popups", "from {0}*/{1}").format(appimage_path, appimage_bin),
+            self.FIELD_APPIMAGE
+        )
+
     def _add_dst_networks_to_combo(self, combo, dst_ip):
         if type(ipaddress.ip_address(dst_ip)) == ipaddress.IPv4Address:
             combo.addItem(QC.translate("popups", "to {0}").format(ipaddress.ip_network(dst_ip + "/24", strict=False)),  self.FIELD_DST_NETWORK)
@@ -481,6 +562,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         the pop-up dialog. Example:
             curl is connecting to www.opensnitch.io on TCP port 443
         """
+        app_name = self.truncate_text(app_name)
         message = "<b>%s</b>" % app_name
         if not self._local:
             message = QC.translate("popups", "<b>Remote</b> process %s running on <b>%s</b>") % ( \
@@ -505,6 +587,9 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 con.protocol.upper(),
                 con.dst_port)
 
+        if self.messageLabel.width() >= self._width:
+            self.messageLabel.setText("\u200b".join(self.messageLabel.text()))
+
         return "%s %s" % (message, msg_action)
 
     def _get_duration(self, duration_idx):
@@ -525,27 +610,27 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         else:
             return Config.DURATION_ALWAYS
 
-    def _get_combo_operator(self, combo, what_idx):
+    def _get_combo_operator(self, combo, what_idx, con):
         if combo.itemData(what_idx) == self.FIELD_PROC_PATH:
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, self._con.process_path
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, con.process_path
 
         elif combo.itemData(what_idx) == self.FIELD_PROC_ARGS:
             # this should not happen
-            if len(self._con.process_args) == 0 or self._con.process_args[0] == "":
-                return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, self._con.process_path
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_COMMAND, ' '.join(self._con.process_args)
+            if len(con.process_args) == 0 or con.process_args[0] == "":
+                return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_PATH, con.process_path
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_COMMAND, ' '.join(con.process_args)
 
         elif combo.itemData(what_idx) == self.FIELD_PROC_ID:
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_ID, "{0}".format(self._con.process_id)
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_PROCESS_ID, "{0}".format(con.process_id)
 
         elif combo.itemData(what_idx) == self.FIELD_USER_ID:
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_USER_ID, "%s" % self._con.user_id
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_USER_ID, "%s" % con.user_id
 
         elif combo.itemData(what_idx) == self.FIELD_DST_PORT:
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_PORT, "%s" % self._con.dst_port
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_PORT, "%s" % con.dst_port
 
         elif combo.itemData(what_idx) == self.FIELD_DST_IP:
-            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_IP, self._con.dst_ip
+            return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_IP, con.dst_ip
 
         elif combo.itemData(what_idx) == self.FIELD_DST_HOST:
             return Config.RULE_TYPE_SIMPLE, Config.OPERAND_DEST_HOST, combo.currentText()
@@ -561,14 +646,20 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             parts = combo.currentText().split(' ')
             text = parts[len(parts)-1]
             # ^(|.*\.)yahoo\.com
-            dsthost = '\.'.join(text.split('.')).replace("*", "")
-            dsthost = "^(|.*\.)%s" % dsthost[2:]
+            dsthost = r'\.'.join(text.split('.')).replace("*", "")
+            dsthost = r'^(|.*\.)%s$' % dsthost[2:]
             return Config.RULE_TYPE_REGEXP, Config.OPERAND_DEST_HOST, dsthost
 
         elif combo.itemData(what_idx) == self.FIELD_REGEX_IP:
             parts = combo.currentText().split(' ')
             text = parts[len(parts)-1]
-            return Config.RULE_TYPE_REGEXP, Config.OPERAND_DEST_IP, "%s" % '\.'.join(text.split('.')).replace("*", ".*")
+            return Config.RULE_TYPE_REGEXP, Config.OPERAND_DEST_IP, "%s" % r'\.'.join(text.split('.')).replace("*", ".*")
+
+        elif combo.itemData(what_idx) == self.FIELD_APPIMAGE:
+            appimage_bin = os.path.basename(con.process_path)
+            appimage_path = os.path.dirname(con.process_path).replace('.', r'\.')
+            appimage_path = appimage_path[0:len(self.APPIMAGE_PREFIX)+7]
+            return Config.RULE_TYPE_REGEXP, Config.OPERAND_PROCESS_PATH, r'^{0}[0-9A-Za-z]{{6}}\/.*{1}$'.format(appimage_path, appimage_bin)
 
     def _on_action_clicked(self, action):
         self._default_action = action
@@ -597,6 +688,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         try:
             self._cfg.setSettings("promptDialog/geometry", self.saveGeometry())
             self._rule = ui_pb2.Rule(name="user.choice")
+            self._rule.created = int(datetime.now().timestamp())
             self._rule.enabled = True
             self._rule.duration = self._get_duration(self.durationCombo.currentIndex())
 
@@ -607,7 +699,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 self._rule.action = Config.ACTION_REJECT
 
             what_idx = self.whatCombo.currentIndex()
-            self._rule.operator.type, self._rule.operator.operand, self._rule.operator.data = self._get_combo_operator(self.whatCombo, what_idx)
+            self._rule.operator.type, self._rule.operator.operand, self._rule.operator.data = self._get_combo_operator(self.whatCombo, what_idx, self._con)
             if self._rule.operator.data == "":
                 print("Invalid rule, discarding: ", self._rule)
                 self._rule = None
@@ -619,7 +711,7 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             # TODO: move to a method
             data=[]
             if self.checkDstIP.isChecked() and self.whatCombo.itemData(what_idx) != self.FIELD_DST_IP:
-                _type, _operand, _data = self._get_combo_operator(self.whatIPCombo, self.whatIPCombo.currentIndex())
+                _type, _operand, _data = self._get_combo_operator(self.whatIPCombo, self.whatIPCombo.currentIndex(), self._con)
                 data.append({"type": _type, "operand": _operand, "data": _data})
                 rule_temp_name = slugify("%s %s" % (rule_temp_name, _data))
 
@@ -634,22 +726,40 @@ class PromptDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             is_list_rule = self._is_list_rule()
 
             # If the user has selected to filter by cmdline, but the launched
-            # command path is not absolute, we can't trust it. In this case,
-            # also filter by the absolute path to the binary.
+            # command path is not absolute or the first component contains
+            # "/proc/" (/proc/self/fd.., /proc/1234/fd...), we can't trust it.
+            # In these cases, also filter by the absolute path to the binary.
             if self._rule.operator.operand == Config.OPERAND_PROCESS_COMMAND:
                 proc_args = " ".join(self._con.process_args)
                 proc_args = proc_args.split(" ")
-                if os.path.isabs(proc_args[0]) == False:
+                if os.path.isabs(proc_args[0]) == False or proc_args[0].startswith("/proc"):
                     is_list_rule = True
                     data.append({"type": Config.RULE_TYPE_SIMPLE, "operand": Config.OPERAND_PROCESS_PATH, "data": str(self._con.process_path)})
 
             if is_list_rule:
-                data.append({"type": self._rule.operator.type, "operand": self._rule.operator.operand, "data": self._rule.operator.data})
+                data.append({
+                    "type": self._rule.operator.type,
+                    "operand": self._rule.operator.operand,
+                    "data": self._rule.operator.data
+                })
+                # We need to send back the operator list to the AskRule() call
+                # as json string, in order to add it to the DB.
                 self._rule.operator.data = json.dumps(data)
                 self._rule.operator.type = Config.RULE_TYPE_LIST
                 self._rule.operator.operand = Config.RULE_TYPE_LIST
+                for op in data:
+                    self._rule.operator.list.extend([
+                        ui_pb2.Operator(
+                            type=op['type'],
+                            operand=op['operand'],
+                            sensitive=False if op.get('sensitive') == None else op['sensitive'],
+                            data="" if op.get('data') == None else op['data']
+                        )
+                    ])
 
-            self._rule.name = rule_temp_name
+            exists = self._rules.exists(self._rule, self._peer)
+            if not exists:
+                self._rule.name = self._rules.new_unique_name(rule_temp_name, self._peer, "")
 
             self.hide()
             if self._ischeckAdvanceded:
