@@ -4,40 +4,52 @@ from PyQt5.QtCore import QCoreApplication as QC
 from slugify import slugify
 from datetime import datetime
 import re
-import json
 import sys
 import os
 import pwd
-from opensnitch import ui_pb2
 import time
 import ipaddress
 
+import opensnitch.proto as proto
+ui_pb2, ui_pb2_grpc = proto.import_()
+
 from opensnitch.config import Config
 from opensnitch.nodes import Nodes
 from opensnitch.database import Database
 from opensnitch.database.enums import RuleFields, ConnFields
 from opensnitch.version import version
-from opensnitch.utils import Message, FileDialog, Icons, NetworkInterfaces
+from opensnitch.utils import (
+    Message,
+    FileDialog,
+    Icons,
+    NetworkInterfaces,
+    qvalidator
+)
 from opensnitch.rules import Rule, Rules
 
 DIALOG_UI_PATH = "%s/../res/ruleseditor.ui" % os.path.dirname(sys.modules[__name__].__file__)
 class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
 
     LOG_TAG = "[rules editor]"
-    classA_net = "10\.\d{1,3}\.\d{1,3}\.\d{1,3}"
-    classB_net = "172\.1[6-9]\.\d+\.\d+|172\.2[0-9]\.\d+\.\d+|172\.3[0-1]+\.\d{1,3}\.\d{1,3}"
-    classC_net = "192\.168\.\d{1,3}\.\d{1,3}"
-    others_net = "127\.\d{1,3}\.\d{1,3}\.\d{1,3}|169\.254\.\d{1,3}\.\d{1,3}"
-    multinets = "2[32][23459]\.\d{1,3}\.\d{1,3}\.\d{1,3}"
+    classA_net = r'10\.\d{1,3}\.\d{1,3}\.\d{1,3}'
+    classB_net = r'172\.1[6-9]\.\d+\.\d+|172\.2[0-9]\.\d+\.\d+|172\.3[0-1]+\.\d{1,3}\.\d{1,3}'
+    classC_net = r'192\.168\.\d{1,3}\.\d{1,3}'
+    others_net = r'127\.\d{1,3}\.\d{1,3}\.\d{1,3}|169\.254\.\d{1,3}\.\d{1,3}'
+    multinets = r'2[32][23459]\.\d{1,3}\.\d{1,3}\.\d{1,3}'
     MULTICAST_RANGE = "^(" + multinets + ")$"
     LAN_RANGES = "^(" + others_net + "|" + classC_net + "|" + classB_net + "|" + classA_net + "|::1|f[cde].*::.*)$"
     LAN_LABEL = "LAN"
     MULTICAST_LABEL = "MULTICAST"
 
+    INVALID_RULE_NAME_CHARS = '/'
+
     ADD_RULE = 0
     EDIT_RULE = 1
     WORK_MODE = ADD_RULE
 
+    PW_USER = 0
+    PW_UID = 2
+
     _notification_callback = QtCore.pyqtSignal(ui_pb2.NotificationReply)
 
     def __init__(self, parent=None, _rule=None, appicon=None):
@@ -53,6 +65,10 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.setupUi(self)
         self.setWindowIcon(appicon)
 
+        self.ruleNameValidator = qvalidator.RestrictChars(RulesEditorDialog.INVALID_RULE_NAME_CHARS)
+        self.ruleNameValidator.result.connect(self._cb_rule_name_validator_result)
+        self.ruleNameEdit.setValidator(self.ruleNameValidator)
+
         self.buttonBox.setStandardButtons(
             QtWidgets.QDialogButtonBox.Help |
             QtWidgets.QDialogButtonBox.Reset |
@@ -90,10 +106,10 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         if QtGui.QIcon.hasThemeIcon("emblem-default"):
             return
 
-        applyIcon = Icons.new("emblem-default")
-        denyIcon = Icons.new("emblem-important")
-        rejectIcon = Icons.new("window-close")
-        openIcon = Icons.new("document-open")
+        applyIcon = Icons.new(self, "emblem-default")
+        denyIcon = Icons.new(self, "emblem-important")
+        rejectIcon = Icons.new(self, "window-close")
+        openIcon = Icons.new(self, "document-open")
         self.actionAllowRadio.setIcon(applyIcon)
         self.actionDenyRadio.setIcon(denyIcon)
         self.actionRejectRadio.setIcon(rejectIcon)
@@ -125,8 +141,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                 self._users_list = pwd.getpwall()
                 self.uidCombo.blockSignals(True);
                 for user in self._users_list:
-                    self.uidCombo.addItem("{0} ({1})".format(user[0], user[2]), user[2])
-                #self.uidCombo.setCurrentText("")
+                    self.uidCombo.addItem("{0} ({1})".format(user[self.PW_USER], user[self.PW_UID]), user[self.PW_UID])
             except Exception as e:
                 print("[ruleseditor] Error adding IPs:", e)
             finally:
@@ -137,6 +152,16 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     def _bool(self, s):
         return s == 'True'
 
+    def _cb_rule_name_validator_result(self, result):
+        if result == QtGui.QValidator.Invalid:
+            self._set_status_error(
+                QC.translate("rules",
+                             "Invalid rule name (not allowed characters: '{0}' )".format(RulesEditorDialog.INVALID_RULE_NAME_CHARS)
+                             )
+            )
+        else:
+            self._set_status_message("")
+
     def _cb_accept_clicked(self):
         pass
 
@@ -221,7 +246,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         self.selectNetsListButton.setEnabled(state)
 
     def _cb_uid_combo_changed(self, index):
-        self.uidCombo.setCurrentText(str(self._users_list[index][2]))
+        self.uidCombo.setCurrentText(str(self._users_list[index][self.PW_UID]))
 
     def _set_status_error(self, msg):
         self.statusLabel.setStyleSheet('color: red')
@@ -334,6 +359,16 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
             self.statusLabel.setText(str(e))
             return False
 
+    def _is_valid_list_path(self, listWidget):
+        if listWidget.text() == "":
+            return QC.translate("rules", "Lists field cannot be empty")
+        if self._nodes.is_local(self.nodesCombo.currentText()) and \
+            self.nodeApplyAllCheck.isChecked() == False and \
+            os.path.isdir(listWidget.text()) == False:
+            return QC.translate("rules", "Lists field must be a directory")
+
+        return None
+
     def set_fields_from_connection(self, records):
         self.nodesCombo.setCurrentText(records.value(ConnFields.Node))
         self.protoCombo.setCurrentText(records.value(ConnFields.Protocol).upper())
@@ -432,13 +467,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         if self.rule.operator.type != Config.RULE_TYPE_LIST:
             self._load_rule_operator(self.rule.operator)
         else:
-            rule_options = json.loads(self.rule.operator.data)
-            for r in rule_options:
-                _sensitive = False
-                if 'sensitive' in r:
-                    _sensitive = r['sensitive']
-
-                op = ui_pb2.Operator(type=r['type'], operand=r['operand'], data=r['data'], sensitive=_sensitive)
+            for op in self.rule.operator.list:
                 self._load_rule_operator(op)
 
         return True
@@ -567,14 +596,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
     def _insert_rule_to_db(self, node_addr):
         # the order of the fields doesn't matter here, as long as we use the
         # name of the field.
-        self._db.insert("rules",
-            "(time, node, name, description, enabled, precedence, nolog, action, duration, operator_type, operator_sensitive, operator_operand, operator_data)",
-                (datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
-                    node_addr, self.rule.name, self.rule.description,
-                    str(self.rule.enabled), str(self.rule.precedence), str(self.rule.nolog),
-                    self.rule.action, self.rule.duration, self.rule.operator.type,
-                    str(self.rule.operator.sensitive), self.rule.operator.operand, self.rule.operator.data),
-                action_on_conflict="REPLACE")
+        self._rules.add_rules(node_addr, [self.rule])
 
     def _add_rule(self):
         try:
@@ -627,6 +649,7 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
         - If the user has not provided a rule name, auto assign one.
         """
         self.rule = ui_pb2.Rule()
+        self.rule.created = int(datetime.now().timestamp())
         self.rule.name = self.ruleNameEdit.text()
         self.rule.description = self.ruleDescEdit.toPlainText()
         self.rule.enabled = self.enableCheck.isChecked()
@@ -847,22 +870,41 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         })
 
         if self.uidCheck.isChecked():
-            if self.uidCombo.currentText() == "":
+            uidType = Config.RULE_TYPE_SIMPLE
+            uid = self.uidCombo.currentText()
+
+            if uid == "":
                 return False, QC.translate("rules", "User ID can not be empty")
 
+            try:
+                # sometimes when loading a rule, instead of the UID, the format
+                # "user (uid)" is set. So try to parse it, in order not to save
+                # a wrong uid.
+                uidtmp = uid.split(" ")
+                if len(uidtmp) == 1:
+                    int(uidtmp[0])
+                else:
+                    uid = str(pwd.getpwnam(uidtmp[0])[self.PW_UID])
+            except:
+                # if it's not a digit and nor a system user (user (id)), see if
+                # it's a regexp.
+                if self._is_regex(self.uidCombo.currentText()):
+                    uidType = Config.RULE_TYPE_REGEXP
+                    if self._is_valid_regex(self.uidCombo.currentText()) == False:
+                        return False, QC.translate("rules", "User ID regexp error")
+
+                else:
+                    return False, QC.translate("rules", "Invalid UID, it must be a digit.")
+
             self.rule.operator.operand = Config.OPERAND_USER_ID
             self.rule.operator.data = self.uidCombo.currentText()
             rule_data.append(
                     {
-                        'type': Config.RULE_TYPE_SIMPLE,
+                        'type': uidType,
                         'operand': Config.OPERAND_USER_ID,
-                        'data': self.uidCombo.currentText(),
+                        'data': uid,
                         "sensitive": self.sensitiveCheck.isChecked()
                         })
-            if self._is_regex(self.uidCombo.currentText()):
-                rule_data[len(rule_data)-1]['type'] = Config.RULE_TYPE_REGEXP
-                if self._is_valid_regex(self.uidCombo.currentText()) == False:
-                    return False, QC.translate("rules", "User ID regexp error")
 
         if self.pidCheck.isChecked():
             if self.pidLine.text() == "":
@@ -883,10 +925,9 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                     return False, QC.translate("rules", "PID field regexp error")
 
         if self.dstListsCheck.isChecked():
-            if self.dstListsLine.text() == "":
-                return False, QC.translate("rules", "Lists field cannot be empty")
-            if os.path.isdir(self.dstListsLine.text()) == False:
-                return False, QC.translate("rules", "Lists field must be a directory")
+            error = self._is_valid_list_path(self.dstListsLine)
+            if error:
+                return False, error
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
             self.rule.operator.operand = Config.OPERAND_LIST_DOMAINS
@@ -897,13 +938,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         'data': self.dstListsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
-            self.rule.operator.data = json.dumps(rule_data)
+            self.rule.operator.data = ""
 
         if self.dstListRegexpCheck.isChecked():
-            if self.dstRegexpListsLine.text() == "":
-                return False, QC.translate("rules", "Lists field cannot be empty")
-            if os.path.isdir(self.dstRegexpListsLine.text()) == False:
-                return False, QC.translate("rules", "Lists field must be a directory")
+            error = self._is_valid_list_path(self.dstRegexpListsLine)
+            if error:
+                return False, error
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
             self.rule.operator.operand = Config.OPERAND_LIST_DOMAINS_REGEXP
@@ -914,13 +954,12 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         'data': self.dstRegexpListsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
-            self.rule.operator.data = json.dumps(rule_data)
+            self.rule.operator.data = ""
 
         if self.dstListNetsCheck.isChecked():
-            if self.dstListNetsLine.text() == "":
-                return False, QC.translate("rules", "Lists field cannot be empty")
-            if os.path.isdir(self.dstListNetsLine.text()) == False:
-                return False, QC.translate("rules", "Lists field must be a directory")
+            error = self._is_valid_list_path(self.dstListNetsLine)
+            if error:
+                return False, error
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
             self.rule.operator.operand = Config.OPERAND_LIST_NETS
@@ -931,14 +970,13 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         'data': self.dstListNetsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
-            self.rule.operator.data = json.dumps(rule_data)
+            self.rule.operator.data = ""
 
 
         if self.dstListIPsCheck.isChecked():
-            if self.dstListIPsLine.text() == "":
-                return False, QC.translate("rules", "Lists field cannot be empty")
-            if os.path.isdir(self.dstListIPsLine.text()) == False:
-                return False, QC.translate("rules", "Lists field must be a directory")
+            error = self._is_valid_list_path(self.dstListIPsLine)
+            if error:
+                return False, error
 
             self.rule.operator.type = Config.RULE_TYPE_LISTS
             self.rule.operator.operand = Config.OPERAND_LIST_IPS
@@ -949,12 +987,22 @@ class RulesEditorDialog(QtWidgets.QDialog, uic.loadUiType(DIALOG_UI_PATH)[0]):
                         'data': self.dstListIPsLine.text(),
                         'sensitive': self.sensitiveCheck.isChecked()
                         })
-            self.rule.operator.data = json.dumps(rule_data)
+            self.rule.operator.data = ""
 
         if len(rule_data) >= 2:
             self.rule.operator.type = Config.RULE_TYPE_LIST
             self.rule.operator.operand = Config.RULE_TYPE_LIST
-            self.rule.operator.data = json.dumps(rule_data)
+            self.rule.operator.data = ""
+            for rd in rule_data:
+                self.rule.operator.list.extend([
+                    ui_pb2.Operator(
+                        type=rd['type'],
+                        operand=rd['operand'],
+                        data=rd['data'],
+                        sensitive=rd['sensitive']
+                    )
+                ])
+                print(self.rule.operator.list)
 
         elif len(rule_data) == 1:
             self.rule.operator.operand = rule_data[0]['operand']
